searchData={"content_type":"text/markdown","items":[{"doc":"Provides functions for encoding and decoding avro records.\n\nKafka has another approach to schemaless, which prepends a schema ID -- to be\ninvestigated; hopefully we can sort out the seeming incompatibilities at some\npoint, once and for all, without too many workarounds.\n\nThe proper approach is either:\n  - prepend C3 01 + 8 byte fingerprint to the binary (Avro single-object\n    encoding)\n  - prepend 00 + 4 byte schema ID (Confluent Schema Registry Wire Format)\n\nFinally, the same message is encoded differently in Python and in Elixir\n(fastavro appears to have trouble roundtripping records). The decoded\nversions largely agree, but have some subtle differences. This needs to be\ninvestigated. It might require some small schema tweaks to avoid this kind of\nsituation.","ref":"Avrogen.html","title":"Avrogen","type":"module"},{"doc":"","ref":"Avrogen.html#decode_schemaless/2","title":"Avrogen.decode_schemaless/2","type":"function"},{"doc":"","ref":"Avrogen.html#decode_schemaless_base64/2","title":"Avrogen.decode_schemaless_base64/2","type":"function"},{"doc":"","ref":"Avrogen.html#encode_schemaless/1","title":"Avrogen.encode_schemaless/1","type":"function"},{"doc":"","ref":"Avrogen.html#encode_schemaless_base64/1","title":"Avrogen.encode_schemaless_base64/1","type":"function"},{"doc":"The entry point for code generation. It starts by taking a raw schema: JSON matching the\n  [specification](https://avro.apache.org/docs/1.11.0/spec.html#schemas).\n\n  An Avro schema is a recursive data structure.","ref":"Avrogen.Avro.Schema.html","title":"Avrogen.Avro.Schema","type":"module"},{"doc":"","ref":"Avrogen.Avro.Schema.html#external_dependencies/1","title":"Avrogen.Avro.Schema.external_dependencies/1","type":"function"},{"doc":"Get the filename of the generated elixir source file from a schema.","ref":"Avrogen.Avro.Schema.html#filename_from_schema/2","title":"Avrogen.Avro.Schema.filename_from_schema/2","type":"function"},{"doc":"Get all the filenames of the generated elixir source file from a schema.","ref":"Avrogen.Avro.Schema.html#filenames_from_schema/3","title":"Avrogen.Avro.Schema.filenames_from_schema/3","type":"function"},{"doc":"This function will generate elixir modules for the provided schema. If the schema\n  references external types (i.e. types that are not embedded within the schema),\n  then the schemas for the external dependencies should be passed as the second\n  argument.\n\n  Overview of code:\n\n  - This function works by first parsing the schemas into `Schema.t()` which is a recursive\n    data structure representing the avro schema file.\n  - It then does a second parse over the schema where it normalizes the schema by extracting\n    all the embedded types and replacing the embedded type with a reference to the schema.\n  - The final stage is generation of the specific modules (for `enum`s and 'record`s).\n    The code generation relies on the 'CodeGenerator` protocol defined below where each\n    part of the schema implements this protocol and is responsible for generating snippets\n    of the generated code. These snippets are combined the `CodeGenerator` implementation in\n    the patent type.","ref":"Avrogen.Avro.Schema.html#generate_code/4","title":"Avrogen.Avro.Schema.generate_code/4","type":"function"},{"doc":"","ref":"Avrogen.Avro.Schema.html#module_path/3","title":"Avrogen.Avro.Schema.module_path/3","type":"function"},{"doc":"","ref":"Avrogen.Avro.Schema.html#normalized_schemas/3","title":"Avrogen.Avro.Schema.normalized_schemas/3","type":"function"},{"doc":"","ref":"Avrogen.Avro.Schema.html#parse/1","title":"Avrogen.Avro.Schema.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Schema.html#t:file_name/0","title":"Avrogen.Avro.Schema.file_name/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Schema.html#t:generated_code/0","title":"Avrogen.Avro.Schema.generated_code/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Schema.html#t:non_union/0","title":"Avrogen.Avro.Schema.non_union/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Schema.html#t:raw_schema/0","title":"Avrogen.Avro.Schema.raw_schema/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Schema.html#t:t/0","title":"Avrogen.Avro.Schema.t/0","type":"type"},{"doc":"Protocol defining how different types representing an Avro schema should contribute\n  to the code generation process.","ref":"Avrogen.Avro.Schema.CodeGenerator.html","title":"Avrogen.Avro.Schema.CodeGenerator","type":"protocol"},{"doc":"Returns whether this type contains any PII including any PII in nested types.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#contains_pii?/2","title":"Avrogen.Avro.Schema.CodeGenerator.contains_pii?/2","type":"function"},{"doc":"Returns the function responsible decoding the value from an avro map.\n\n  Note the returned function should have a return type as a result tuple.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#decode_function/3","title":"Avrogen.Avro.Schema.CodeGenerator.decode_function/3","type":"function"},{"doc":"Returns the function for dropping the the PII values stored in this type.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#drop_pii/3","title":"Avrogen.Avro.Schema.CodeGenerator.drop_pii/3","type":"function"},{"doc":"Returns the elixir type spec for this Avro schema type.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#elixir_type/1","title":"Avrogen.Avro.Schema.CodeGenerator.elixir_type/1","type":"function"},{"doc":"Returns the function responsible for encoding the value to an avro map\n\n  This should return a function with an infallible return type","ref":"Avrogen.Avro.Schema.CodeGenerator.html#encode_function/3","title":"Avrogen.Avro.Schema.CodeGenerator.encode_function/3","type":"function"},{"doc":"Extract the external dependencies fro this schema.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#external_dependencies/1","title":"Avrogen.Avro.Schema.CodeGenerator.external_dependencies/1","type":"function"},{"doc":"Normalizes the schema by extracting all embedded schemas into `global` and returning the\n  schema with the embedded types replaced with references to their global names.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#normalize/4","title":"Avrogen.Avro.Schema.CodeGenerator.normalize/4","type":"function"},{"doc":"Returns the snippet of code responsible for generating a random instance of this type.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#random_instance/3","title":"Avrogen.Avro.Schema.CodeGenerator.random_instance/3","type":"function"},{"doc":"","ref":"Avrogen.Avro.Schema.CodeGenerator.html#t:global/0","title":"Avrogen.Avro.Schema.CodeGenerator.global/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Schema.CodeGenerator.html#t:parent_namespace/0","title":"Avrogen.Avro.Schema.CodeGenerator.parent_namespace/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Schema.CodeGenerator.html#t:scoped_embedded_types/0","title":"Avrogen.Avro.Schema.CodeGenerator.scoped_embedded_types/0","type":"type"},{"doc":"All the types that implement this protocol.","ref":"Avrogen.Avro.Schema.CodeGenerator.html#t:t/0","title":"Avrogen.Avro.Schema.CodeGenerator.t/0","type":"type"},{"doc":"This type is a representation of the [Avro array type](https://avro.apache.org/docs/1.11.0/spec.html#Arrays).","ref":"Avrogen.Avro.Types.Array.html","title":"Avrogen.Avro.Types.Array","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Array.html#identifier/0","title":"Avrogen.Avro.Types.Array.identifier/0","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Array.html#parse/1","title":"Avrogen.Avro.Types.Array.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Array.html#t:t/0","title":"Avrogen.Avro.Types.Array.t/0","type":"type"},{"doc":"This type is a representation of the [Avro Enum type](https://avro.apache.org/docs/1.11.0/spec.html#Enums).","ref":"Avrogen.Avro.Types.Enum.html","title":"Avrogen.Avro.Types.Enum","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Enum.html#constructor_functions/1","title":"Avrogen.Avro.Types.Enum.constructor_functions/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Enum.html#fullname/2","title":"Avrogen.Avro.Types.Enum.fullname/2","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Enum.html#generate_module/2","title":"Avrogen.Avro.Types.Enum.generate_module/2","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Enum.html#namespace/3","title":"Avrogen.Avro.Types.Enum.namespace/3","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Enum.html#parse/1","title":"Avrogen.Avro.Types.Enum.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Enum.html#t:t/0","title":"Avrogen.Avro.Types.Enum.t/0","type":"type"},{"doc":"This module delegates the parsing logic to the specific Logical types modules.\n\n  [Logical types](https://avro.apache.org/docs/1.11.0/spec.html#Logical+Types)\n\n  A logical type is an Avro primitive or complex type with extra attributes to represent\n  a derived type. The attribute logicalType must always be present for a logical type, and\n  is a string with the name of one of the logical types listed later in this section.\n  Other attributes may be defined for particular logical types.\n\n  A logical type is always serialized using its underlying Avro type so that values are\n  encoded in exactly the same way as the equivalent Avro type that does not have a\n  logicalType attribute. Language implementations may choose to represent logical types\n  with an appropriate native type, although this is not required.\n\n  Language implementations must ignore unknown logical types when reading, and should use\n  the underlying Avro type. If a logical type is invalid, for example a decimal with scale\n  greater than its precision, then implementations should ignore the logical type and use\n  the underlying Avro type.","ref":"Avrogen.Avro.Types.Logical.html","title":"Avrogen.Avro.Types.Logical","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.html#module/1","title":"Avrogen.Avro.Types.Logical.module/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.html#parse/1","title":"Avrogen.Avro.Types.Logical.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.html#t:t/0","title":"Avrogen.Avro.Types.Logical.t/0","type":"type"},{"doc":"This type represents the [date](https://avro.apache.org/docs/1.11.1/specification/_print/#date)\n  type according to the specification.\n\n  The date logical type represents a date within the calendar, with no reference to a particular time zone or time of\n  day.\n\n  A date logical type annotates an Avro int, where the int stores the number of days from the unix epoch, 1 January\n  1970 (ISO calendar).","ref":"Avrogen.Avro.Types.Logical.Date.html","title":"Avrogen.Avro.Types.Logical.Date","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.Date.html#parse/1","title":"Avrogen.Avro.Types.Logical.Date.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.Date.html#t:t/0","title":"Avrogen.Avro.Types.Logical.Date.t/0","type":"type"},{"doc":"This type actually does not conform to the specification for a date.\n\n  However, this is in use in our code base. This type is when a string represents a\n  date value as stored as an iso8601 string.","ref":"Avrogen.Avro.Types.Logical.DateString.html","title":"Avrogen.Avro.Types.Logical.DateString","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.DateString.html#parse/1","title":"Avrogen.Avro.Types.Logical.DateString.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.DateString.html#t:t/0","title":"Avrogen.Avro.Types.Logical.DateString.t/0","type":"type"},{"doc":"This type actually does not conform to the specification for a decimal.\n\n  However, this is in use in our code base. This type is when a string represents a\n  date time value as stored as an iso8601 string.","ref":"Avrogen.Avro.Types.Logical.DateTimeString.html","title":"Avrogen.Avro.Types.Logical.DateTimeString","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.DateTimeString.html#parse/1","title":"Avrogen.Avro.Types.Logical.DateTimeString.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.DateTimeString.html#t:t/0","title":"Avrogen.Avro.Types.Logical.DateTimeString.t/0","type":"type"},{"doc":"This represents the logical type of the [decimal](https://avro.apache.org/docs/1.11.1/specification/_print/#decimal)\n  in the specification\n\n  The decimal logical type represents an arbitrary-precision signed decimal number of\n  the form unscaled × 10-scale.\n\n  Officially this typs can be either bytes or fixed, currently I have not added support for fixed.\n\n  The following attributes are supported:\n\n  - scale, a JSON integer representing the scale (optional).\n    If not specified the scale is 0.\n  - precision, a JSON integer representing the (maximum) precision of decimals\n    stored in this type (required).\n\n  Precision must be a positive integer greater than zero. If the underlying type is a\n  fixed, then the precision is limited by its size. An array of length n can store at\n  most floor(log_10(28 × n - 1 - 1)) base-10 digits of precision.\n\n  Scale must be zero or a positive integer less than or equal to the precision.\n\n  For the purposes of schema resolution, two schemas that are decimal logical types\n  match if their scales and precisions match.","ref":"Avrogen.Avro.Types.Logical.Decimal.html","title":"Avrogen.Avro.Types.Logical.Decimal","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.Decimal.html#number_of_bytes/1","title":"Avrogen.Avro.Types.Logical.Decimal.number_of_bytes/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.Decimal.html#parse/1","title":"Avrogen.Avro.Types.Logical.Decimal.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.Decimal.html#t:t/0","title":"Avrogen.Avro.Types.Logical.Decimal.t/0","type":"type"},{"doc":"This type actually does not conform to the specification for a decimal.\n\n  However, this is in use in our code base. This type is when a string represents a\n  decimal value.","ref":"Avrogen.Avro.Types.Logical.DecimalString.html","title":"Avrogen.Avro.Types.Logical.DecimalString","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.DecimalString.html#parse/1","title":"Avrogen.Avro.Types.Logical.DecimalString.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.DecimalString.html#t:t/0","title":"Avrogen.Avro.Types.Logical.DecimalString.t/0","type":"type"},{"doc":"This type represents the [local-timestamp-micros](https://avro.apache.org/docs/1.11.1/specification/_print/#local-timestamp-microsecond-precision)\n  type according to the specification\n\n  The timestamp-micros logical type represents a time of day, with no reference to a particular\n  calendar, time zone or date, with a precision of one microsecond.\n\n  A timestamp-micros logical type annotates an Avro long, where the long stores the number of\n  microseconds after midnight, 00:00:00.000.","ref":"Avrogen.Avro.Types.Logical.LocalTimestampMicros.html","title":"Avrogen.Avro.Types.Logical.LocalTimestampMicros","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.LocalTimestampMicros.html#parse/1","title":"Avrogen.Avro.Types.Logical.LocalTimestampMicros.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.LocalTimestampMicros.html#t:t/0","title":"Avrogen.Avro.Types.Logical.LocalTimestampMicros.t/0","type":"type"},{"doc":"This type represents the [local-timestamp-millis](https://avro.apache.org/docs/1.11.1/specification/_print/#local-timestamp-millisecond-precision)\n  type according to the specification\n\n  The local-timestamp-millis logical type represents a time of day, with no reference to a particular\n  calendar, time zone or date, with a precision of one millisecond.\n\n  A local-timestamp-millis logical type annotates an Avro long, where the long stores the number of\n  milliseconds after midnight, 00:00:00.000.","ref":"Avrogen.Avro.Types.Logical.LocalTimestampMillis.html","title":"Avrogen.Avro.Types.Logical.LocalTimestampMillis","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.LocalTimestampMillis.html#parse/1","title":"Avrogen.Avro.Types.Logical.LocalTimestampMillis.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.LocalTimestampMillis.html#t:t/0","title":"Avrogen.Avro.Types.Logical.LocalTimestampMillis.t/0","type":"type"},{"doc":"This type represents the [time-micros](https://avro.apache.org/docs/1.11.1/specification/_print/#time-microsecond-precision)\n  type according to the specification\n\n  The time-micros logical type represents a time, with no reference to a particular calendar or time zone,\n  with a precision of one microsecond.\n\n  A time-micros logical type annotates an Avro long, where the int stores the number of\n  microseconds after midnight, 00:00:00.000.","ref":"Avrogen.Avro.Types.Logical.TimeMicros.html","title":"Avrogen.Avro.Types.Logical.TimeMicros","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimeMicros.html#parse/1","title":"Avrogen.Avro.Types.Logical.TimeMicros.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimeMicros.html#t:t/0","title":"Avrogen.Avro.Types.Logical.TimeMicros.t/0","type":"type"},{"doc":"This type represents the [time-millis](https://avro.apache.org/docs/1.11.1/specification/_print/#time-millisecond-precision)\n  type according to the specification\n\n  The time-millis logical type represents a time, with no reference to a particular calendar or time zone,\n  with a precision of one millisecond.\n\n  A time-millis logical type annotates an Avro int, where the int stores the number of\n  milliseconds after midnight, 00:00:00.000.","ref":"Avrogen.Avro.Types.Logical.TimeMillis.html","title":"Avrogen.Avro.Types.Logical.TimeMillis","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimeMillis.html#parse/1","title":"Avrogen.Avro.Types.Logical.TimeMillis.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimeMillis.html#t:t/0","title":"Avrogen.Avro.Types.Logical.TimeMillis.t/0","type":"type"},{"doc":"This type represents the [timestamp-micros](https://avro.apache.org/docs/1.11.1/specification/#timestamp-microsecond-precision)\n  type according to the specification\n\n  The timestamp-micros logical type represents a date with a time and a reference to a particular\n  time zone, with a precision of one microsecond.\n\n  A timestamp-micros logical type annotates an Avro long, where the long stores the number of\n  microseconds after midnight, 00:00:00.000.","ref":"Avrogen.Avro.Types.Logical.TimestampMicros.html","title":"Avrogen.Avro.Types.Logical.TimestampMicros","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimestampMicros.html#parse/1","title":"Avrogen.Avro.Types.Logical.TimestampMicros.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimestampMicros.html#t:t/0","title":"Avrogen.Avro.Types.Logical.TimestampMicros.t/0","type":"type"},{"doc":"This type represents the [timestamp-millis](https://avro.apache.org/docs/1.11.1/specification/_print/#timestamp-millisecond-precision)\n  type according to the specification\n\n  The timestamp-millis logical type represents an instant on the global timeline, independent of a particular time\n  zone or calendar, with a precision of one millisecond.\n\n  A timestamp-millis logical type annotates an Avro long, where the long stores the number of\n  milliseconds after midnight, 00:00:00.000.","ref":"Avrogen.Avro.Types.Logical.TimestampMillis.html","title":"Avrogen.Avro.Types.Logical.TimestampMillis","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimestampMillis.html#parse/1","title":"Avrogen.Avro.Types.Logical.TimestampMillis.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.TimestampMillis.html#t:t/0","title":"Avrogen.Avro.Types.Logical.TimestampMillis.t/0","type":"type"},{"doc":"This represents the logical type of the [uuid](https://avro.apache.org/docs/1.11.1/specification/_print/#uuid)\n  in the specification.\n\n  The uuid logical type represents a random generated universally unique identifier (UUID).\n\n  A uuid logical type annotates an Avro string. The string has to conform with RFC-4122","ref":"Avrogen.Avro.Types.Logical.UUID.html","title":"Avrogen.Avro.Types.Logical.UUID","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.UUID.html#parse/1","title":"Avrogen.Avro.Types.Logical.UUID.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Logical.UUID.html#t:t/0","title":"Avrogen.Avro.Types.Logical.UUID.t/0","type":"type"},{"doc":"This type is a representation of the [Avro map type](https://avro.apache.org/docs/1.11.0/spec.html#Maps).","ref":"Avrogen.Avro.Types.Map.html","title":"Avrogen.Avro.Types.Map","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Map.html#identifier/0","title":"Avrogen.Avro.Types.Map.identifier/0","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Map.html#parse/1","title":"Avrogen.Avro.Types.Map.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Map.html#t:t/0","title":"Avrogen.Avro.Types.Map.t/0","type":"type"},{"doc":"This type is a representation of the [Avro array type](https://avro.apache.org/docs/1.11.0/spec.html#schema_primitive).","ref":"Avrogen.Avro.Types.Primitive.html","title":"Avrogen.Avro.Types.Primitive","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#default_value/1","title":"Avrogen.Avro.Types.Primitive.default_value/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#encode_type/1","title":"Avrogen.Avro.Types.Primitive.encode_type/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#guard_clause/1","title":"Avrogen.Avro.Types.Primitive.guard_clause/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#is_primitive/1","title":"Avrogen.Avro.Types.Primitive.is_primitive/1","type":"macro"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#null/0","title":"Avrogen.Avro.Types.Primitive.null/0","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#parse/1","title":"Avrogen.Avro.Types.Primitive.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#parse_type/1","title":"Avrogen.Avro.Types.Primitive.parse_type/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#t:primitive_type/0","title":"Avrogen.Avro.Types.Primitive.primitive_type/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Types.Primitive.html#t:t/0","title":"Avrogen.Avro.Types.Primitive.t/0","type":"type"},{"doc":"This type is a representation of the [Avro record type](https://avro.apache.org/docs/1.11.0/spec.html#Records).","ref":"Avrogen.Avro.Types.Record.html","title":"Avrogen.Avro.Types.Record","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Record.html#fullname/2","title":"Avrogen.Avro.Types.Record.fullname/2","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.html#generate_module/4","title":"Avrogen.Avro.Types.Record.generate_module/4","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.html#identifier/0","title":"Avrogen.Avro.Types.Record.identifier/0","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.html#namespace/3","title":"Avrogen.Avro.Types.Record.namespace/3","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.html#parse/1","title":"Avrogen.Avro.Types.Record.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.html#t:t/0","title":"Avrogen.Avro.Types.Record.t/0","type":"type"},{"doc":"This type is a representation of the [Avro record field type](https://avro.apache.org/docs/1.11.0/spec.html#Record).","ref":"Avrogen.Avro.Types.Record.Field.html","title":"Avrogen.Avro.Types.Record.Field","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#bind_with_default/1","title":"Avrogen.Avro.Types.Record.Field.bind_with_default/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#comment/1","title":"Avrogen.Avro.Types.Record.Field.comment/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#decode_function_name/1","title":"Avrogen.Avro.Types.Record.Field.decode_function_name/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#drop_pii_function_name/1","title":"Avrogen.Avro.Types.Record.Field.drop_pii_function_name/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#encode_function_name/1","title":"Avrogen.Avro.Types.Record.Field.encode_function_name/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#enforce?/1","title":"Avrogen.Avro.Types.Record.Field.enforce?/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#from_avro_map_clause/1","title":"Avrogen.Avro.Types.Record.Field.from_avro_map_clause/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#has_default?/1","title":"Avrogen.Avro.Types.Record.Field.has_default?/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#is_pii?/1","title":"Avrogen.Avro.Types.Record.Field.is_pii?/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#name/1","title":"Avrogen.Avro.Types.Record.Field.name/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#parse/1","title":"Avrogen.Avro.Types.Record.Field.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#parse_default_value/1","title":"Avrogen.Avro.Types.Record.Field.parse_default_value/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#range_opts/1","title":"Avrogen.Avro.Types.Record.Field.range_opts/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#to_avro_map_clause/1","title":"Avrogen.Avro.Types.Record.Field.to_avro_map_clause/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#to_keywords/1","title":"Avrogen.Avro.Types.Record.Field.to_keywords/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#typed_struct_field/1","title":"Avrogen.Avro.Types.Record.Field.typed_struct_field/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#t:order/0","title":"Avrogen.Avro.Types.Record.Field.order/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Types.Record.Field.html#t:t/0","title":"Avrogen.Avro.Types.Record.Field.t/0","type":"type"},{"doc":"A reference to an external schema name.","ref":"Avrogen.Avro.Types.Reference.html","title":"Avrogen.Avro.Types.Reference","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Reference.html#elixir_type_name/1","title":"Avrogen.Avro.Types.Reference.elixir_type_name/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Reference.html#from_name/1","title":"Avrogen.Avro.Types.Reference.from_name/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Reference.html#parse/1","title":"Avrogen.Avro.Types.Reference.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Reference.html#t:t/0","title":"Avrogen.Avro.Types.Reference.t/0","type":"type"},{"doc":"This type is a representation of the [Avro union type](https://avro.apache.org/docs/1.11.0/spec.html#Unions).\n\n  (Note that when a default value is specified for a record field whose type is a union, the type of\n  the default value must match the first element of the union. Thus, for unions containing \"null\",\n  the \"null\" is usually listed first, since the default value of such unions is typically null.)\n\n  Unions may not contain more than one schema with the same type, except for the named types record,\n  fixed and enum. For example, unions containing two array types or two map types are not permitted,\n  but two types with different names are permitted. (Names permit efficient resolution when reading\n  and writing unions.)\n\n  Unions may not immediately contain other unions.","ref":"Avrogen.Avro.Types.Union.html","title":"Avrogen.Avro.Types.Union","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Union.html#has_member?/2","title":"Avrogen.Avro.Types.Union.has_member?/2","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Union.html#parse/1","title":"Avrogen.Avro.Types.Union.parse/1","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Union.html#t:t/0","title":"Avrogen.Avro.Types.Union.t/0","type":"type"},{"doc":"Shared utils for working with Avro schema types","ref":"Avrogen.Avro.Types.Utils.html","title":"Avrogen.Avro.Types.Utils","type":"module"},{"doc":"","ref":"Avrogen.Avro.Types.Utils.html#fullname/2","title":"Avrogen.Avro.Types.Utils.fullname/2","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Utils.html#namespace/3","title":"Avrogen.Avro.Types.Utils.namespace/3","type":"function"},{"doc":"","ref":"Avrogen.Avro.Types.Utils.html#t:namespaced_type/0","title":"Avrogen.Avro.Types.Utils.namespaced_type/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Types.Utils.html#t:parent_namespace/0","title":"Avrogen.Avro.Types.Utils.parent_namespace/0","type":"type"},{"doc":"","ref":"Avrogen.Avro.Types.Utils.html#t:scoped_embedded_types/0","title":"Avrogen.Avro.Types.Utils.scoped_embedded_types/0","type":"type"},{"doc":"Behaviour for generated record modules.","ref":"Avrogen.AvroModule.html","title":"Avrogen.AvroModule","type":"behaviour"},{"doc":"The fully qualified avro name of this record type.","ref":"Avrogen.AvroModule.html#c:avro_fqn/0","title":"Avrogen.AvroModule.avro_fqn/0","type":"callback"},{"doc":"The name of the schema that defines the record represented by this module.","ref":"Avrogen.AvroModule.html#c:avro_schema_name/0","title":"Avrogen.AvroModule.avro_schema_name/0","type":"callback"},{"doc":"Turn a raw map from an erlavro decoding operation into (nested) structs as\ndefined by the generated avro modules.","ref":"Avrogen.AvroModule.html#c:from_avro_map/1","title":"Avrogen.AvroModule.from_avro_map/1","type":"callback"},{"doc":"Whether embedded schemas should be scoped to the type they are defined in.\n\nFor example, for the following schema\n\n```json\n{\n  \"name\": \"Event\",\n  \"namespace\": \"events\",\n  \"type\": \"record\",\n  \"fields\": [\n    {\n      \"name\": \"details\",\n      \"type\": {\n        \"name\": \"Subtype\",\n        \"type\": \"record\",\n        \"fields\": [\n          ...\n        ]\n      }\n    }\n  ]\n}\n```\n\nIf set to `true` the internal type would be called \"Events.Event.Subtype\", otherwise, it would be\n\"Events.Subtype\".\n\nIf not provided default to `false`","ref":"Avrogen.AvroModule.html#c:scope_embedded_types?/0","title":"Avrogen.AvroModule.scope_embedded_types?/0","type":"callback"},{"doc":"Turn a (nested) struct (as defined by the generated avro modules) into an\nintermediate map that can be passed to a suitable erlavro encoder.","ref":"Avrogen.AvroModule.html#c:to_avro_map/1","title":"Avrogen.AvroModule.to_avro_map/1","type":"callback"},{"doc":"Sigil for the `Decimal` type.","ref":"Avrogen.Decimal.Sigil.html","title":"Avrogen.Decimal.Sigil","type":"module"},{"doc":"","ref":"Avrogen.Decimal.Sigil.html#sigil_d/2","title":"Avrogen.Decimal.Sigil.sigil_d/2","type":"function"},{"doc":"Utils for extracting various info from avro schemas.\nSchemas can be records or enums.","ref":"Avrogen.Schema.html","title":"Avrogen.Schema","type":"module"},{"doc":"Get a list of external types referenced in a given record/enum.","ref":"Avrogen.Schema.html#external_dependencies/1","title":"Avrogen.Schema.external_dependencies/1","type":"function"},{"doc":"Get a schema's fully qualified name by combining its namespace and name.","ref":"Avrogen.Schema.html#fqn/1","title":"Avrogen.Schema.fqn/1","type":"function"},{"doc":"Load a schema from a file.","ref":"Avrogen.Schema.html#load_schema!/1","title":"Avrogen.Schema.load_schema!/1","type":"function"},{"doc":"Get a schema's name.","ref":"Avrogen.Schema.html#name/1","title":"Avrogen.Schema.name/1","type":"function"},{"doc":"Get a schema's namespace.\n\nNote: Unsure what to do if the schema has no namespace - make it a MatchError for now...","ref":"Avrogen.Schema.html#namespace/1","title":"Avrogen.Schema.namespace/1","type":"function"},{"doc":"Resolve a schema's file path from its fqn.","ref":"Avrogen.Schema.html#path_from_fqn/3","title":"Avrogen.Schema.path_from_fqn/3","type":"function"},{"doc":"Sort a list of avro schemas in topological order based on dependencies.","ref":"Avrogen.Schema.html#topological_sort/1","title":"Avrogen.Schema.topological_sort/1","type":"function"},{"doc":"","ref":"Avrogen.Schema.html#t:schema/0","title":"Avrogen.Schema.schema/0","type":"type"},{"doc":"Common behaviour for modules that define avro schemas in code.","ref":"Avrogen.Schema.SchemaModule.html","title":"Avrogen.Schema.SchemaModule","type":"behaviour"},{"doc":"The actual schema, but as an elixir list, so it can be 'imported' into other\nschemas.","ref":"Avrogen.Schema.SchemaModule.html#c:avro_schema_elixir/0","title":"Avrogen.Schema.SchemaModule.avro_schema_elixir/0","type":"callback"},{"doc":"The actual schema, i.e. a json-string.","ref":"Avrogen.Schema.SchemaModule.html#c:avsc/0","title":"Avrogen.Schema.SchemaModule.avsc/0","type":"callback"},{"doc":"The schema's name, used for creating a file name.","ref":"Avrogen.Schema.SchemaModule.html#c:schema_name/0","title":"Avrogen.Schema.SchemaModule.schema_name/0","type":"callback"},{"doc":"Stores avro schemas and corresponding codec functions; provides lookup by\nschema name. Initialized on startup with schemas from priv directory.","ref":"Avrogen.Schema.SchemaRegistry.html","title":"Avrogen.Schema.SchemaRegistry","type":"module"},{"doc":"Expand a wildcard relative to an app's root directory.","ref":"Avrogen.Schema.SchemaRegistry.html#app_wildcard/2","title":"Avrogen.Schema.SchemaRegistry.app_wildcard/2","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Avrogen.Schema.SchemaRegistry.html#child_spec/1","title":"Avrogen.Schema.SchemaRegistry.child_spec/1","type":"function"},{"doc":"A decoder hook that will convert erlavro :null values to nil.","ref":"Avrogen.Schema.SchemaRegistry.html#decoder_hook/4","title":"Avrogen.Schema.SchemaRegistry.decoder_hook/4","type":"function"},{"doc":"Return the super-schema which contains the topologically sorted concatenation\nof all schemas managed by this registry as a json string.","ref":"Avrogen.Schema.SchemaRegistry.html#get_avsc/0","title":"Avrogen.Schema.SchemaRegistry.get_avsc/0","type":"function"},{"doc":"Return the binary decoding function which is capable of decoding all schemas.","ref":"Avrogen.Schema.SchemaRegistry.html#get_decoder/0","title":"Avrogen.Schema.SchemaRegistry.get_decoder/0","type":"function"},{"doc":"Return the binary encoder function which is capable of encoding all messages.","ref":"Avrogen.Schema.SchemaRegistry.html#get_encoder/0","title":"Avrogen.Schema.SchemaRegistry.get_encoder/0","type":"function"},{"doc":"","ref":"Avrogen.Schema.SchemaRegistry.html#make_decoder/1","title":"Avrogen.Schema.SchemaRegistry.make_decoder/1","type":"function"},{"doc":"","ref":"Avrogen.Schema.SchemaRegistry.html#make_encoder/1","title":"Avrogen.Schema.SchemaRegistry.make_encoder/1","type":"function"},{"doc":"","ref":"Avrogen.Schema.SchemaRegistry.html#start_link/1","title":"Avrogen.Schema.SchemaRegistry.start_link/1","type":"function"},{"doc":"Generate avro schemas from .exs files.","ref":"Avrogen.SchemaGenerator.html","title":"Avrogen.SchemaGenerator","type":"module"},{"doc":"Generates the avro schemas for a given .exs file, and returns a list of the\nschemas generated.","ref":"Avrogen.SchemaGenerator.html#generate_avsc_files!/3","title":"Avrogen.SchemaGenerator.generate_avsc_files!/3","type":"function"},{"doc":"Utils for extracting various info from avro schemas types.","ref":"Avrogen.Types.html","title":"Avrogen.Types","type":"module"},{"doc":"Get a list of types referenced in a given type, inluding any primitives.","ref":"Avrogen.Types.html#dependencies/1","title":"Avrogen.Types.dependencies/1","type":"function"},{"doc":"Get a list of external types referenced in a given type.","ref":"Avrogen.Types.html#external_dependencies/1","title":"Avrogen.Types.external_dependencies/1","type":"function"},{"doc":"","ref":"Avrogen.Types.html#is_primitive/1","title":"Avrogen.Types.is_primitive/1","type":"macro"},{"doc":"Returns true if the given type is a primitive.","ref":"Avrogen.Types.html#is_primitive?/1","title":"Avrogen.Types.is_primitive?/1","type":"function"},{"doc":"Fuzzy enum matching.","ref":"Avrogen.Util.FuzzyEnumMatch.html","title":"Avrogen.Util.FuzzyEnumMatch","type":"module"},{"doc":"Returns the closest match for s from the lookup index; returns the default\nvalue if the similarity is less than the given threshold or if s is not a\nstring.","ref":"Avrogen.Util.FuzzyEnumMatch.html#best_match/4","title":"Avrogen.Util.FuzzyEnumMatch.best_match/4","type":"function"},{"doc":"","ref":"Avrogen.Util.FuzzyEnumMatch.html#candidates/2","title":"Avrogen.Util.FuzzyEnumMatch.candidates/2","type":"function"},{"doc":"","ref":"Avrogen.Util.FuzzyEnumMatch.html#make_index/1","title":"Avrogen.Util.FuzzyEnumMatch.make_index/1","type":"function"},{"doc":"","ref":"Avrogen.Util.FuzzyEnumMatch.html#preprocess/1","title":"Avrogen.Util.FuzzyEnumMatch.preprocess/1","type":"function"},{"doc":"Helper functions for generating random values.","ref":"Avrogen.Util.Random.html","title":"Avrogen.Util.Random","type":"module"},{"doc":"Returns a tuple {new_rand_state, b}, where b is a random boolean.","ref":"Avrogen.Util.Random.html#boolean/1","title":"Avrogen.Util.Random.boolean/1","type":"function"},{"doc":"Returns updated rand_state and random date between given start and end date.","ref":"Avrogen.Util.Random.html#date/3","title":"Avrogen.Util.Random.date/3","type":"function"},{"doc":"Picks a random date/datetime d such that start_date <= d < end_date. Returns\n{new_rand_state, d}.","ref":"Avrogen.Util.Random.html#datetime/3","title":"Avrogen.Util.Random.datetime/3","type":"function"},{"doc":"Returns a tuple {new_rand_state, d}, where d is a decimal with min <= d <\nmax.","ref":"Avrogen.Util.Random.html#decimal/4","title":"Avrogen.Util.Random.decimal/4","type":"function"},{"doc":"Exports the rand_state in a simple form that can be passed into\ninit_rand_state/1.","ref":"Avrogen.Util.Random.html#export_rand_state/1","title":"Avrogen.Util.Random.export_rand_state/1","type":"function"},{"doc":"Returns a tuple {new_rand_state, f}, where f is a float with min <= f < max.","ref":"Avrogen.Util.Random.html#float/3","title":"Avrogen.Util.Random.float/3","type":"function"},{"doc":"Creates a fresh rand_state.","ref":"Avrogen.Util.Random.html#init_rand_state/0","title":"Avrogen.Util.Random.init_rand_state/0","type":"function"},{"doc":"Recreates a particular rand_state from the output of export_rand_state/1.","ref":"Avrogen.Util.Random.html#init_rand_state/1","title":"Avrogen.Util.Random.init_rand_state/1","type":"function"},{"doc":"Returns a tuple {new_rand_state, i}, where min <= i < max.","ref":"Avrogen.Util.Random.html#integer/3","title":"Avrogen.Util.Random.integer/3","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.html#list_element/2","title":"Avrogen.Util.Random.list_element/2","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.html#postcode/1","title":"Avrogen.Util.Random.postcode/1","type":"function"},{"doc":"Creates an infinite stream of random avro instances for a given generated\navro record module. Useful for 'fuzz-testing' anything that processes avro\ndata: push large numbers of random instances through the code and see if\nanything raises an exception.\n\nEach element in the stream is a tuple {rand_state, instance}, where\nrand_state is the random number generator state that was used to create the\ninstance.  Feeding this rand_state into the AvroRecordModule's\nrandom_instance function will re-create the instance.","ref":"Avrogen.Util.Random.html#stream/1","title":"Avrogen.Util.Random.stream/1","type":"function"},{"doc":"Creates a random stream with a particular starting rand_state, so an entire\nstream can be replayed just from the seed.","ref":"Avrogen.Util.Random.html#stream/2","title":"Avrogen.Util.Random.stream/2","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.html#string/4","title":"Avrogen.Util.Random.string/4","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.html#time/3","title":"Avrogen.Util.Random.time/3","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.html#t:rand_state/0","title":"Avrogen.Util.Random.rand_state/0","type":"type"},{"doc":"Provides functions that return \"constructor functions\". A constructor\nfunction is a function that expects a rand_state and optional parameters,\nand, on invocation, returns an updated rand_state and a randomly generated\nvalue.\n\nThe functions in this module are basically 'lazy' wrappers around eager\nfunctions in the parent module.\n\nThanks to this 'laziness', we can specify lists of alternative constructor\nfunctions, from which we can randomly choose an element for evaluation\n(without forcing all elements to be evaluated up front).","ref":"Avrogen.Util.Random.Constructors.html","title":"Avrogen.Util.Random.Constructors","type":"module"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#boolean/0","title":"Avrogen.Util.Random.Constructors.boolean/0","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#date/1","title":"Avrogen.Util.Random.Constructors.date/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#datetime/1","title":"Avrogen.Util.Random.Constructors.datetime/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#decimal/1","title":"Avrogen.Util.Random.Constructors.decimal/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#enum_value/1","title":"Avrogen.Util.Random.Constructors.enum_value/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#float/1","title":"Avrogen.Util.Random.Constructors.float/1","type":"function"},{"doc":"Instantiates a struct with random values as per the given constructors\nlist. Returns the updated rand_state and the instantiated struct.","ref":"Avrogen.Util.Random.Constructors.html#instantiate/3","title":"Avrogen.Util.Random.Constructors.instantiate/3","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#integer/1","title":"Avrogen.Util.Random.Constructors.integer/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#list/2","title":"Avrogen.Util.Random.Constructors.list/2","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#map/2","title":"Avrogen.Util.Random.Constructors.map/2","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#nothing/0","title":"Avrogen.Util.Random.Constructors.nothing/0","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#string/1","title":"Avrogen.Util.Random.Constructors.string/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#time/1","title":"Avrogen.Util.Random.Constructors.time/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#uuid/1","title":"Avrogen.Util.Random.Constructors.uuid/1","type":"function"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#t:constructor_fun/0","title":"Avrogen.Util.Random.Constructors.constructor_fun/0","type":"type"},{"doc":"","ref":"Avrogen.Util.Random.Constructors.html#t:constructor_list/0","title":"Avrogen.Util.Random.Constructors.constructor_list/0","type":"type"},{"doc":"Utilities for working with code generation for the Avro schemas","ref":"Avrogen.Utils.MacroUtils.html","title":"Avrogen.Utils.MacroUtils","type":"module"},{"doc":"This function flattens blocks of code together to avoid additional nesting and strange\n  behaviour in generated code.","ref":"Avrogen.Utils.MacroUtils.html#flatten_block/1","title":"Avrogen.Utils.MacroUtils.flatten_block/1","type":"function"},{"doc":"This macro generated the implementation of Jason.Encoder which skips the encoding of\n  `nil` valued fields.","ref":"Avrogen.Utils.MacroUtils.html#jason_decode_skip_null_impl/1","title":"Avrogen.Utils.MacroUtils.jason_decode_skip_null_impl/1","type":"macro"},{"doc":"Compiler task to generate elixir code from avro schemas.","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html","title":"mix compile.avro_code_generator","type":"task"},{"doc":"Use this task by adding it to the list of compilers in your mix.exs for your project, and add\nconfiguration options using the `avro_code_generator_opts` key in your project options.\n\nExample configuration:\n```\ndef project do\n  [\n    ...\n    compilers: [:avro_code_generator | Mix.compilers()],\n    ...\n    avro_code_generator_opts: [\n      paths: [\"priv/schema/*.avsc\"],\n      dest: \"generated/\",\n      scoped_embed_paths: [\"priv/schema/events*.avsc\"],\n      schema_root: \"priv/schema\",\n      module_prefix: \"Avro.Generated\"\n    ]\n    ...\n  ]\nend\n```\n\nWhere:\n\n- paths: A list of wildcards used to locate schema files to generate code for. Each path is\n    evaluated with Path.wildcard().\n- dest: Where to put the generated elixir files.\n- schema_root: Where to find external schema files (see notes on schema file naming below)\n- module_prefix: A common prefix prepended to the front of all module names.\n- scoped_embed_paths: the glob patterns of the files where any embedded scopes should have\n  the generated module path contain the encompasing types.\n\n  For example, for the following schema\n\n  ```json\n  {\n    \"name\": \"Event\",\n    \"namespace\": \"events\",\n    \"type\": \"record\",\n    \"fields\": [\n      {\n        \"name\": \"details\",\n        \"type\": {\n          \"name\": \"Subtype\",\n          \"type\": \"record\",\n          \"fields\": [\n            ...\n          ]\n        }\n      }\n    ]\n  }\n  ```\n  If this file is included in the scoped_embed_paths, then the generated module for `Subtype`\n  would be called `Events.Event.Subtype` otherwise it would be `Events.Subtype `. This option\n  is useful when you have naming clashes in embedded schema subtypes, or if you simply want to\n  namespace subtypes to avoid potential future clashes","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html#module-usage","title":"Usage - mix compile.avro_code_generator","type":"task"},{"doc":"This task requires schema files to contain one schema only (nested schemas are allowed). Where\nreferences are made to external schemas, it must be able to be found by looking for the schema by\nname in the directory supplied by the `schema_root` option, and the name of the schema files must\nfollow the specific pattern:\n\n```\n / . .avsc\n```\n\nE.g. `root/foo.bar/Baz.avsc`","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html#module-schema-file-naming","title":"Schema File Naming - mix compile.avro_code_generator","type":"task"},{"doc":"Generated files are named like so:\n\n```\n / / .ex\n```\n\nE.g. `dest/foo/bar/baz.ex\n\nNote, the namespace is split on periods into directories, and the schema name is converted from\ncamel case into snake case.","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html#module-generated-file-naming","title":"Generated file naming - mix compile.avro_code_generator","type":"task"},{"doc":"In order to make the build as fast as possible, this task only generates files when it needs to,\ni.e. when the schemas have changed, or when the code of the avro_compiler app has changed.\n\nEach avsc file will generate one or more elixir source code file, but the content of the generated\ncode may depend on other avsc files if the schema references external schemas from other files.\n\nFor example, the dependency tree of the generated file foo/Bar.ex might look like this:\n\nfoo.Bar.avsc <─┬── foo/Bar.ex\nfoo.Baz.avsc <─┤\nfoo.Qux.avsc <─┘\n\nArrows point in the direction of the dependency, e.g. foo/Bar.ex depends on foo.Bar.avsc,\nfoo.Baz.avsc, and foo.Qux.avsc. This means that the content of foo/Bar.ex depends on the content\nall three avsc files, so it must be re-generated if any of the avsc files are modified.","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html#module-dependency-tracking","title":"Dependency Tracking - mix compile.avro_code_generator","type":"task"},{"doc":"In order to keep a clean dest directory, each time the task is run, the dest dir (supplied in\nthe options) is wiped clean. The generated files are generated into a directory somewhere inside\nthe _build directory, and then the relevant files are copied over to the dest dir. This ensures\nno files are left behind in the dest dir which could be picked up by the elixir compiler. Thus,\nit's important that this dest dir does not contain other important files as these will be wiped\nout.","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html#module-cleaning-the-destination-directory","title":"Cleaning the Destination Directory - mix compile.avro_code_generator","type":"task"},{"doc":"This task writes a manifest file containing a the config options and a list of generated files\neach time it's run. The task then uses this manifest file to work out when the options have\nchanged so it can trigger a full re-build.\n\nThis manifest could also used to know which files to delete when running a clean operation, but\nunfortunately running mix clean when this app is in an umbrella doesn't work. This is because the\ncode for this task is removed before the clean operation has a chance to run, which eems like a\nmajor flaw in the way tasks work). Hopefully one day the Mix devs will fix this, or we could move\nthis app into a separate repo and make it an external dependency of stonehenge at which point the\nclean operation would work as mix clean does not clean external deps.","ref":"Mix.Tasks.Compile.AvroCodeGenerator.html#module-manifest-file","title":"Manifest File - mix compile.avro_code_generator","type":"task"},{"doc":"Manifest file (see above).","ref":"Mix.Tasks.Compile.AvroCodeGenerator.Manifest.html","title":"mix compile.avro_code_generator.manifest","type":"task"},{"doc":"","ref":"Mix.Tasks.Compile.AvroCodeGenerator.Manifest.html#t:t/0","title":"Mix.Tasks.Compile.AvroCodeGenerator.Manifest.t/0","type":"type"},{"doc":"Compiler task to generate avro schemas from templates defined in elixir.","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.html","title":"mix compile.avro_schema_generator","type":"task"},{"doc":"Use this task by adding it to the list of compilers in your mix.exs for your project, and add\nconfiguration options using the `avro_schema_generator_opts` key in your project options.\n\nExample configuration:\n```\ndef project do\n  [\n    ...\n    compilers: [:avro_schema_generator | Mix.compilers()],\n    ...\n    avro_schema_generator_opts: [\n      paths: [\"schema/**/*.exs\"],\n      dest: \"priv/schema/\"\n    ]\n    ...\n  ]\nend\n```\n\nWhere:\n\n- paths: A list of wildcards used to locate elixir template files to generate code for. Each path\n    is evaluated with Path.wildcard().\n- dest: Where to put the generated schema files.","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.html#module-usage","title":"Usage - mix compile.avro_schema_generator","type":"task"},{"doc":"Schema files are generated one schema per file, where the name of each file is described below.\n\n```\n / . .avsc\n```\n\nE.g. `root/foo.bar/Baz.avsc`","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.html#module-schema-file-naming","title":"Schema File Naming - mix compile.avro_schema_generator","type":"task"},{"doc":"In order to make the build as fast as possible, it is important that this task only generates\nfiles when it needs to, i.e. when sources have changed thus, proper dependency management is\nrequired.\n\nEach template file (exs) will generate one or more schema file (avsc), one for each schema it\ndefines. Thus each generated schema file will depend only on its source exs file.\n\nExample dependency tree:\n\nfoo.exs <─┬─ foo.Bar.avsc\n          ├─ foo.Baz.avsc\n          └─ foo.Qux.avsc\n\nArrows point in the direction of the dependency, e.g. foo.Bar.avsc depends on foo.exs. This means\nthat the content of foo.Bar.avsc depends entirely on the content of foo.exs, so it must be updated\nonly if foo.exs is newer than it.\n\nBecause it's not obvious what ex files would be generated (without just running the generation),\neach time we run the task we store the list of generated files in a manifest file. The next time\nwe run the task, the list of  generated files from the previous run is recalled by reading the\nmanifest file, and this list is used to work out if any of the generated files are older than any\nof their dependencies, thus triggering a re-gen of just those files.\n\nThis manifest could also used to know which files to delete when running a clean operation, but\nunfortunately running mix clean when this app is in an umbrella doesn't work. I think this is\nbecause this task is \"cleaned\" before it has a chance to run, which effectively wipes out this\ntask entirely. If we made this app an external dependency, it wouldn't be cleaned with mix clean\nand thus will work just fine.","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.html#module-dependency-tracking","title":"Dependency Tracking - mix compile.avro_schema_generator","type":"task"},{"doc":"Defines lists of tasks by the filenames of their outputs, and their dependencies (i.e. inputs).\nIn this case, all tasks have one input (the exs schema template file) and one or more outputs\n(each of the generated avsc files).\nWe also store the task's options, to detect config changes which might affect the output files.","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.Manifest.html","title":"mix compile.avro_schema_generator.manifest","type":"task"},{"doc":"","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.Manifest.html#t:t/0","title":"Mix.Tasks.Compile.AvroSchemaGenerator.Manifest.t/0","type":"type"},{"doc":"Defines a list of sources (i.e. input files) and targets (i.e. output files) for a given task.","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.TaskSummary.html","title":"mix compile.avro_schema_generator.task_summary","type":"task"},{"doc":"","ref":"Mix.Tasks.Compile.AvroSchemaGenerator.TaskSummary.html#t:t/0","title":"Mix.Tasks.Compile.AvroSchemaGenerator.TaskSummary.t/0","type":"type"},{"doc":"# Avrogen\n\n[![Build Status](https://github.com/primait/avrogen/actions/workflows/elixir.yml/badge.svg)](https://github.com/primait/avrogen/actions/workflows/elixir.yml)\n[![Hex.pm](https://img.shields.io/badge/hex.pm-green)](https://hex.pm/packages/prima/avrogen)\n[![Documentation](https://img.shields.io/badge/documentation-gray)](https://prima.hexdocs.pm/avrogen/)\n\nGenerate Elixir typedstructs and various useful helper functions from AVRO\nschemas at compile time.","ref":"readme.html","title":"Avrogen","type":"extras"},{"doc":"While there exists a handful of libraries to encode and decode AVRO messages in\nElixir, all of them consume schemas at runtime, which has the advantage of\nflexibilty e.g. this approach can be used with a schema registry, but you lose\nthe any compile time type safety for your types.\n\nAvrogen generates Elixir code from AVRO schemas, turning each record into module\ncontaining a `typedstruct` and a bunch of helper functions to encode and decode\nthe struct to and from AVRO binary format.\n\nFor example, the following schema...\n\n```json\n{\n  \"type\": \"record\",\n  \"namespace\": \"foo\",\n  \"name\": \"Bar\",\n  \"fields\": [\n    { \"name\": \"baz\", \"type\": [\"null\", \"string\"] },\n    { \"name\": \"qux\", \"type\": \"int\" }\n  ]\n}\n```\n\n... generates a module `foo/Bar.ex` which (with documentation and various bits\nof implementation omitted for the sake of brevity) looks like this:\n\n```elixir\ndefmodule Avro.Foo.Bar do\n  use TypedStruct\n  \n  @expected_keys MapSet.new([\"baz\", \"qux\"])\n  @pii_fields MapSet.new([])\n\n  typedstruct do\n    field :baz, nil | String.t()\n    field :qux, integer(), enforce: true\n  end\n\n  def avro_fqn(), do: \"foo.Bar\"\n  def to_avro_map(...) do ... end\n  def from_avro_map(...) do ... end\n  def pii_fields(), do: @pii_fields\n  def drop_pii(...) do ... end\n  def random_instance(rand_state) do ... end\nend\n```\n\nThe main feature here is the `typedstruct`, which allows us to initialize this\nmodule using the struct syntax:\n\n```elixir\n%Avro.Foo.Bar{\n  baz: \"quux\",\n  qux: 12\n}\n```\n\nThe other helper functions provide extra functionality which are explained\nbelow.","ref":"readme.html#rationale","title":"Rationale - Avrogen","type":"extras"},{"doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `avrogen` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:avrogen, \"~> 0.4.3\", organization: \"prima\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Avrogen","type":"extras"},{"doc":"The easiest way to use avrogen is to add `:avro_code_generator` to your list of\ncompilers in your `mix.exs` file, making sure to place it before the other\ncompilers so all Elixir code is in place before the Elixir compiler runs.\n\n```elixir\ncompilers: [:avro_code_generator | Mix.compilers()]\n```\n\nYou'll also need to tell the elixir compiler to build the generated code, which\ncan be acheived by adding the `generated` dir (the default destination\ndirectory) to your `elixirc_paths`.\n\n```elixir\nelixirc_paths: [\"lib\", \"generated\"]\n```\n\nNow, you can create a new directory called `schemas` at the root of your project\nand put some `.avsc` files in there. They will be built and compiled whenever\nthings need to get recompiled, so just run your mix commands as usual.","ref":"readme.html#usage","title":"Usage - Avrogen","type":"extras"},{"doc":"While the defaults might be OK for some folks, you can configure the generator\ntask from your mix.exs file, using the `avro_code_generator_opts` key.\n\nE.g.\n\n```elixir\navro_code_generator_opts: [\n  paths: [\"schemas/*.avsc\"],\n  dest: \"generated\",\n  schema_root: \"schema\",\n  module_prefix: \"Avro\",\n  scoped_embed_paths: [\"priv/schema/events.*.avsc\"],\n  schema_resolution_mode: :flat\n]\n```\n\nThe options are:\n\n- `paths` - an array of file paths or wildcards to locate schema files. Defaults\n  to `\"schemas/*.avsc\"`.\n- `dest` - A directory of where to put the generated elixir code. Defaults to\n  `\"generated\"`.\n- `schema_root` - The root of the schema directory, this is the root dir that\n  will be used to resolve schemas located in other files. Defaults to\n  `\"schemas\"`\n- `module_prefix` - String to place at the front of generated elixir modules.\n  Defaults to `\"Avro\"`.\n- `schema_resolution_mode` - Tells the code generator how to resolve external\n  schemas to a filename. Defaults to `:flat`.\n- `scope_embed_paths` - the glob patterns of the files where any embedded scopes\n  should have the generated module path contain the encompasing types.\n\n  For example, for the following schema\n\n  ```json\n  {\n    \"name\": \"Event\",\n    \"namespace\": \"events\",\n    \"type\": \"record\",\n    \"fields\": [\n      {\n        \"name\": \"details\",\n        \"type\": {\n          \"name\": \"Subtype\",\n          \"type\": \"record\",\n          \"fields\": [\n            ...\n          ]\n        }\n      }\n    ]\n  }\n  ```\n\n  If this file is included in the scoped_embed_paths, then the generated module\n  for `Subtype` would be called `Events.Event.Subtype` otherwise it would be\n  `Events.Subtype`. This option is useful when you have naming clashes in\n  embedded schema subtypes, or if you simply want to namespace subtypes to avoid\n  potential future clashes","ref":"readme.html#options","title":"Options - Avrogen","type":"extras"},{"doc":"Firstly, you'll need to start the Schema Registry process by adding the\nfollowing entry to your Application file:\n\n```elixir\n@impl true\ndef start(_type, _args) do\n  children = [\n    ...\n    # Start a schema registry\n    {Avrogen.Schema.SchemaRegistry, Application.get_application(__MODULE__)},\n    ...\n  ]\n  ...\nend\n```\n\nNow you can create new records in code using the full module name, which is\ncomprised of your prefix + the namespace + name of the record.\n\nE.g. the record:\n\n```json\n{\n  \"namespace\": \"foo\",\n  \"name\": \"Bar\",\n  \"fields\": [\n    {\"name\": \"quz\", \"type\": ...}\n  ]\n}\n```\n\n... will result in a module called `Avro.Foo.Bar`, which can be used like any\nother normal struct:\n\n```elixir\nmessage = %Avro.Foo.Bar{quz: ...}\n```\n\nEncode this module to a binary using the `Avrogen.encode_schemaless/1` function:\n\n```elixir\n{:ok, bytes} = Avrogen.encode_schemaless(message)\n```\n\nYou can decode it back into a struct using the `Avrogen.decode_schemaless/2`\nfunction, mind that you'll need to pass in the module name as avro binaries\ndon't encode their type.\n\n```elixir\n{:ok, message} = Avrogen.decode_schemaless(Avro.Foo.Bar, bytes)\n```","ref":"readme.html#using-the-generated-code","title":"Using the generated code - Avrogen","type":"extras"},{"doc":"Each generated module comes with the `@derive Jason.Encoder` attribute, which\ntells JSON that the struct can be encoded by simply serializing everything other\nthan the `__struct__` field. See https://hexdocs.pm/jason/Jason.Encoder.html for\nmore details.\n\nThus, converting a message to JSON is as simple as:\n\n```elixir\niex> message = %Avro.Test.Person{name: \"Dave\", age: 37}\niex> Jason.encode(message)\n{:ok, \"{\\\"age\\\":37,\\\"name\\\":\\\"Dave\\\"}\"}\n```\n\n> Note: This is not the same as the official AVRO JSON encoding spec, and is\n> mainly used for debugging / making messages human readable.","ref":"readme.html#converting-to-json","title":"Converting to JSON - Avrogen","type":"extras"},{"doc":"Schemas commonly depend on other schemas, which can be located in a different\nfile.\n\nThe code generator has two different modes for file resolution: tree mode and\nflat mode.\n\nBoth modes work from a root directory passed in via the `schema_root` code\ngenerator option (see above).\n\nIn `:flat` mode, schemas are expected to be a flat list of files in the root dir\nlike so:\n\n```\nname.space.SchemaName -> root/name.space.SchemaName.avsc\n```\n\nThis is how libraries like python's `fastavro` expect schemas to be laid out.\n\nIn `:tree` mode, the namespace is split into directories like so:\n\n```\nname.space.SchemaName -> root/name/space/SchemaName.avsc\n```\n\nThis is how libraries like `avrora` expect schemas to be laid out.","ref":"readme.html#external-schema-resolution","title":"External Schema Resolution - Avrogen","type":"extras"},{"doc":"The table below lists supported primitive AVRO types, and their corresponding\nElixir type:\n\n| AVRO Type | Elixir Type                    |\n| --------- | ------------------------------ |\n| `null`    | `nil`                          |\n| `int`     | `integer`                      |\n| `double`  | `float`                        |\n| `string`  | `String.t()`                   |\n| `bytes`   | `binary`                       |\n| `array`   | `list`                         |\n| `map`     | `%{ String.t() => Value.t() }` |\n\nThe following logical types are also supported:\n\n| AVRO Logical Type        | AVRO Underlying Type | Elixir Type     |\n| ------------------------ | -------------------- | --------------- |\n| `uuid`                   | `string`             | `String`        |\n| `big_decimal`            | `string`             | `Decimal`       |\n| `decimal`                | `string`             | `Decimal`       |\n| `decimal`                | `bytes`              | `Decimal`       |\n| `date`                   | `int`                | `Date`          |\n| `date`                   | `string`             | `Date`          |\n| `iso_date`               | `string`             | `Date`          |\n| `datetime`               | `string`             | `DateTime`      |\n| `iso_datetime`           | `string`             | `DateTime`      |\n| `time-millis`            | `int`                | `Time`          |\n| `time-micros`            | `long`               | `Time`          |\n| `timestamp-millis`       | `long`               | `DateTime`      |\n| `timestamp-micros`       | `long`               | `DateTime`      |\n| `local-timestamp-millis` | `long`               | `NaiveDateTime` |\n| `local-timestamp-micros` | `long`               | `NaiveDateTime` |\n\nThe following AVRO types are not supported (yet):\n\n- `float` (use `double`)\n- `long` (use `int`)\n- `fixed`","ref":"readme.html#types","title":"Types - Avrogen","type":"extras"},{"doc":"Avrogen introduces an unofficial extension to AVRO schema specification which\ncan be used to mark record's fields as PII (Personally Identifiable\nInformation). Each generated record module gets a `drop_pii/1` function which\nrecursively strips away all fields marked as PII in the record, and any records\ncontained within.\n\nMark a field as PII by adding `pii: true` option to the field. For example\nimagine you are storing names and ages of people, and the name is PII (but the\nage isn't).\n\n```json\n{\n  \"type\": \"record\",\n  \"name\": \"Person\",\n  \"namespace\": \"example\",\n  \"fields\": [\n    { \"name\": \"name\", \"type\": [\"null\", \"string\"], \"pii\": true },\n    { \"name\": \"age\", \"type\": \"int\" }\n  ]\n}\n```\n\nThen you can simply call `drop_pii/1` on your record to replace all the PII\nfields with `nil` like so:\n\n```elixir\nex> person = %Avro.Example.Person{name: \"John Smith\", age: 38} \n%Avro.Example.Person{age: 38, name: \"John Smith\"}\n\nex> Avro.Example.Person.drop_pii(person)\n%Avro.Example.Person{age: 38, name: nil}\n```\n\n> Note: Fields marked as PII must be of a union type containing a null.\n\nThe AVRO spec specifies that any extra fields in schemas are ignored, so schemas\ncontaining this extension are backwards compatible with other AVRO parsers, as\nthey will just ignore this field.","ref":"readme.html#pii-fields","title":"PII Fields - Avrogen","type":"extras"},{"doc":"Each generated module contains a function to create a random instance of the\nrecord/enum. This can be useful for fuzz testing, among other things.\n\nE.g. Using the `Person` example above, the generated module will contain the\nfollowing function:\n\n```elixir\ndef random_instance(rand_state) do\n  # ...\nend\n```\n\n> The function expects to be given an erlang random state type object, which can\n> be seeded in one of many ways depending on what you want to do with it. The\n> simplest way to create this random state is to generate it with the default\n> generator - `:rand.seed(:default)`, as demonstrated below.\n\nYou can use this `random_instance/1` function to generate random instances of\nthe module's struct, for example:\n\n```elixir\niex> state = :rand.seed(:default)\niex> {state, person} = Avro.Example.Person.random_instance(state)\niex> person\n%Avro.Foo.Bar{\n  name: <<29, 120, 54, 75, 84, 54, 70, 29, 48, 68, 87, 87>>,\n  age: 1812334491\n}\n```\n\nIn this example, `person` is a random instance of the Avro.Example.Person\nrecord, and `state` is the mutated state which can be used again to pass to the\nnext call to `random_instance/1`.\n\nThe various types produce random values according to the following rules:\n\n| Type                                          | Rule                                                                                                                |\n| --------------------------------------------- | ------------------------------------------------------------------------------------------------------------------- |\n| `null`                                        | Always produces `nil`.                                                                                              |\n| `union`                                       | Random instance of any of the types within the union, where each type is equally likely to be chosen.               |\n| `string`                                      | Random utf8 binary of up to 1000 codepoints, where each codepoint lies in the range `0 <= codepoint < 10,000`.      |\n| `int`, `double`, `big_decimal` (logical type) | Random value in the range `-2,147,483,648 <= value < 2,147,483,648`.                                                |\n| `iso_date` and `iso_datetime` (logical types) | Random value in the range `1970-01-01T00:00:00 <= value < 2045-01-01T00:00:00`.                                     |\n| `enum`                                        | One of the symbols selected at random, with each symbol having an equal probability of showing up.                  |\n| `array`                                       | Random list of up to 10 elements, where the value of each element is a random instance of the array's element type. |\n| `record`                                      | Random instance of that record, where each field of the record is generated randomly according to the above rules.  |\n\nYou can control how random you really want these random instances to be using\nsome more unofficial extensions to the avro spec. For example, you can specify\nthe max and min values of int type fields using the \"range\" specifier like so:\n\n```json\n{\n  \"name\": \"age\",\n  \"type\": \"int\",\n  \"range\": {\n    \"int\": {\n      \"max\": 80,\n      \"min\": 16\n    }\n  }\n}\n```\n\nNow when you call `random_instance/1`, the age field will be limited to the\nrange `16 <= age < 80`.\n\nStrings can be formatted according to semantic formatting. Currently the only\nsupported type is \"postcode\", but support for more types may well be added in\nthe future.\n\nE.g.\n\n```json\n{\n  \"name\": \"postcode\",\n  \"type\": \"string\",\n  \"range\": {\n    \"string\": {\n      \"semantic_type\": \"postcode\"\n    }\n  }\n}\n```\n\nNow, the postcode field will be limited to random postcodes (e.g. `BS23 7SX`),\nrather than completely random strings.","ref":"readme.html#random-instance-generators","title":"Random Instance Generators - Avrogen","type":"extras"},{"doc":"AVRO's avsc format is not always the easiest format to maintain. Because it uses\nJSON, variables and comments are not allowed. Thus, `avrogen` comes with a\nschema generator tool to help ease the process.\n\nThis tool can be optionally used by adding it to the list of compilers for your\nproject's `mix.exs` file like so:\n\n```elixir\ncompilers: [:avro_schema_generator | Mix.compilers()]\n```\n\nNote: If you are also using the `avro_code_generator`, then you will need to put\nthe schema generator before the code generator, as the code generator requires\nschemas in order to do its job.\n\nThen, also in your `mix.exs` file, configure the code generator using the\nfollowing options:\n\n```elixir\navro_schema_generator_opts: [\n  paths: [\"exs_schemas/**/*.exs\"],\n  dest: \"schemas\",\n  schema_resolution_mode: :flat\n],\n```\n\nWhere the options are as follows:\n\n- `paths`: A wildcard expression which matches the location of your schema\n  definition files. Defaults to `exs_schemas/**/*.exs`.\n- `dest`: Where to put generated schema files. Defaults to `schemas`.\n- `schema_resolution_mode`: How to structure the dest dir (see the schema\n  resolution section above). Options are `:flat` or `:tree`. Defaults to\n  `:flat`.\n\nSo what goes in these schema definition files? All files should contain a single\nmodule which implements the `Avrogen.Schema.SchemaModule` behaviour. For\nexample:\n\n```elixir\ndefmodule Person do\n  alias Avrogen.Schema.SchemaModule\n  @behaviour SchemaModule\n  @impl SchemaModule\n  def schema_name(), do: \"application_data.v2\"\n\n  @impl SchemaModule\n  def avsc(), do: avro_schema()\n\n  # \"type\": \"record\",\n  # \"name\": \"Person\",\n  # \"namespace\": \"example\",\n  # \"fields\": [\n  #   {\"name\": \"name\", \"type\": [\"null\", \"string\"], \"pii\": true},\n  #   {\"name\": \"age\", \"type\": \"int\"}\n  # ]\n\n  @person %{\n    type: :record,\n    name: \"Person \",\n    namespace: \"example\",\n    doc: \"Describes a person.\",\n    fields: [\n      %{\n        name: :name,\n        type: [:null, :string],\n        doc: \"\"\"\n        The name of the person.\n        \"\"\"\n      },\n      %{\n        name: :age,\n        type: :int,\n        doc: \"\"\"\n        The age of the person.\n        \"\"\"\n      }\n    ]\n  }\n\n  @avro_spec [\n    @person\n  ]\n\n  def avro_schema() do\n    Jason.encode!(@avro_spec)\n  end\n\n  @impl SchemaModule\n  def avro_schema_elixir() do\n    @avro_spec\n  end\nend\n```\n\nWhen you next compile the code with e.g. `mix compile`, the following avsc\nschema will be generated:\n\n```json\n{\n  \"doc\": \"Describes a person.\",\n  \"fields\": [\n    {\n      \"doc\": \"The name of the person.\\n\",\n      \"name\": \"name\",\n      \"type\": [\n        \"null\",\n        \"string\"\n      ]\n    },\n    {\n      \"doc\": \"The age of the person.\\n\",\n      \"name\": \"age\",\n      \"type\": \"int\"\n    }\n  ],\n  \"name\": \"Person \",\n  \"namespace\": \"example\",\n  \"type\": \"record\"\n}\n```\n\nThere's not much magic here, but it should be evident how elixir variables and\nconstructs can be used to reduce repetition in the definitions of the schemas.\nIt's worth noting that these schema definitions are used to define lists of\nschemas in one go. Each individual schema is pulled out and placed into the\ntarget destination, and the file name is structured like so:\n` / . .avsc`, which is the appropriate format for the avro\ncode generator to use later down the line.\n\nNote that once you enable this tool, it completely takes over the `dest`\ndirectory, so any other files found in here will most likely be removed.","ref":"readme.html#schema-generation","title":"Schema Generation - Avrogen","type":"extras"},{"doc":"Avrora is an Elixir library for encoding/decoding avro messages, with options to\nintegrate with a schema registry.\n\nAvrora can work in conjunction with avrogen quite nicely, with avrogen\ngenerating the elixir code, and avrora handling communication with the schema\nregistry and encoding/decoding of messages.\n\nAvrogen expects schemas to be stored in the filesystem in a \"tree\" style format,\nso make sure to set the option `schema_resolution_mode` to `:tree` for both\ngenerators. Once you have configured the avrora cache (see docs on their\n[README](https://hexdocs.pm/avrora/readme.html#usage)), you can then use\navrogen's typedstructs to create the messages and do some basic type/key\nchecking, and avrora to encode/decode the messages.\n\nFor example, to encode...\n\n```elixir\n%module{} = message = %Avro.Test.Person{name: \"John Smith\", age: 38}\nname = module.avro_fqn()\nmap = module.to_avro_map(message)\n{:ok, bytes} = Avrora.encode(map, schema_name: name)\n# Do something with the bytes\n```\n\n... and then to decode ...\n\n```elixir\n{:ok, [decoded]} = Avrora.decode(bytes, schema_name: Avro.Test.Person.avro_fqn())\n{:ok, message} = Avro.Test.Person.from_avro_map(decoded)\n# Do something with the message\n```\n\n> Note: Of course, this assumes you know the decoded message type ahead of time.\n> You can ask Avrora to infer the message type using magic headers by calling\n> `decode/1` rather than `decode/2` (omitting the `schema_name` option), but it\n> doesn't disclose the inferred schema name to the caller, which is not\n> particularly useful.","ref":"readme.html#using-with-avrora","title":"Using with Avrora - Avrogen","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}